import enum
from typing import (
    TYPE_CHECKING,
    Any,
    Callable,
    Dict,
    List,
    Optional,
    Tuple,
    Type,
    Union,
)

import requests

import pyobas
from pyobas import base
from pyobas import exceptions as exc
from pyobas import utils

__all__ = [
    "GetMixin",
    "GetWithoutIdMixin",
]

if TYPE_CHECKING:
    # When running mypy we use these as the base classes
    _RestManagerBase = base.RESTManager
    _RestObjectBase = base.RESTObject
else:
    _RestManagerBase = object
    _RestObjectBase = object


class HeadMixin(_RestManagerBase):
    @exc.on_http_error(exc.OpenBASHeadError)
    def head(
        self, id: Optional[Union[str, int]] = None, **kwargs: Any
    ) -> "requests.structures.CaseInsensitiveDict[Any]":
        if TYPE_CHECKING:
            assert self.path is not None

        path = self.path
        if id is not None:
            path = f"{path}/{utils.EncodedId(id)}"

        return self.openbas.http_head(path, **kwargs)


class GetMixin(HeadMixin, _RestManagerBase):
    _computed_path: Optional[str]
    _from_parent_attrs: Dict[str, Any]
    _obj_cls: Optional[Type[base.RESTObject]]
    _optional_get_attrs: Tuple[str, ...] = ()
    _parent: Optional[base.RESTObject]
    _parent_attrs: Dict[str, Any]
    _path: Optional[str]
    openbas: pyobas.OpenBAS

    @exc.on_http_error(exc.OpenBASGetError)
    def get(self, id: Union[str, int], **kwargs: Any) -> base.RESTObject:
        if isinstance(id, str):
            id = utils.EncodedId(id)
        path = f"{self.path}/{id}"
        if TYPE_CHECKING:
            assert self._obj_cls is not None
        server_data = self.openbas.http_get(path, **kwargs)
        if TYPE_CHECKING:
            assert not isinstance(server_data, requests.Response)
        return self._obj_cls(self, server_data)


class GetWithoutIdMixin(HeadMixin, _RestManagerBase):
    _computed_path: Optional[str]
    _from_parent_attrs: Dict[str, Any]
    _obj_cls: Optional[Type[base.RESTObject]]
    _optional_get_attrs: Tuple[str, ...] = ()
    _parent: Optional[base.RESTObject]
    _parent_attrs: Dict[str, Any]
    _path: Optional[str]
    openbas: pyobas.OpenBAS

    @exc.on_http_error(exc.OpenBASGetError)
    def get(self, **kwargs: Any) -> base.RESTObject:
        if TYPE_CHECKING:
            assert self.path is not None
        server_data = self.openbas.http_get(self.path, **kwargs)
        if TYPE_CHECKING:
            assert not isinstance(server_data, requests.Response)
            assert self._obj_cls is not None
        return self._obj_cls(self, server_data)


class ListMixin(HeadMixin, _RestManagerBase):
    _computed_path: Optional[str]
    _from_parent_attrs: Dict[str, Any]
    _list_filters: Tuple[str, ...] = ()
    _obj_cls: Optional[Type[base.RESTObject]]
    _parent: Optional[base.RESTObject]
    _parent_attrs: Dict[str, Any]
    _path: Optional[str]
    openbas: pyobas.OpenBAS

    @exc.on_http_error(exc.OpenBASListError)
    def list(self, **kwargs: Any) -> Union[base.RESTObjectList, List[base.RESTObject]]:

        if self.openbas.per_page:
            kwargs.setdefault("per_page", self.openbas.per_page)

        # global keyset pagination
        if self.openbas.pagination:
            kwargs.setdefault("pagination", self.openbas.pagination)

        if self.openbas.order_by:
            kwargs.setdefault("order_by", self.openbas.order_by)

        # Allow to overwrite the path, handy for custom listings
        path = kwargs.pop("path", self.path)

        if TYPE_CHECKING:
            assert self._obj_cls is not None
        obj = self.openbas.http_list(path, **kwargs)
        if isinstance(obj, list):
            return [self._obj_cls(self, item, created_from_list=True) for item in obj]
        return base.RESTObjectList(self, self._obj_cls, obj)


@enum.unique
class UpdateMethod(enum.IntEnum):
    PUT = 1
    POST = 2
    PATCH = 3


class UpdateMixin(_RestManagerBase):
    _computed_path: Optional[str]
    _from_parent_attrs: Dict[str, Any]
    _obj_cls: Optional[Type[base.RESTObject]]
    _parent: Optional[base.RESTObject]
    _parent_attrs: Dict[str, Any]
    _path: Optional[str]
    _update_method: UpdateMethod = UpdateMethod.PUT
    openbas: pyobas.OpenBAS

    def _get_update_method(
        self,
    ) -> Callable[..., Union[Dict[str, Any], requests.Response]]:
        """Return the HTTP method to use.

        Returns:
            http_put (default) or http_post
        """
        if self._update_method is UpdateMethod.POST:
            http_method = self.openbas.http_post
        elif self._update_method is UpdateMethod.PATCH:
            # only patch uses required kwargs, so our types are a bit misaligned
            http_method = self.openbas.http_patch  # type: ignore[assignment]
        else:
            http_method = self.openbas.http_put
        return http_method

    @exc.on_http_error(exc.OpenBASUpdateError)
    def update(
        self,
        id: Optional[Union[str, int]] = None,
        new_data: Optional[Dict[str, Any]] = None,
        **kwargs: Any,
    ) -> Dict[str, Any]:
        new_data = new_data or {}

        if id is None:
            path = self.path
        else:
            path = f"{self.path}/{utils.EncodedId(id)}"

        # excludes = []
        # if self._obj_cls is not None and self._obj_cls._id_attr is not None:
        #    excludes = [self._obj_cls._id_attr]
        # self._update_attrs.validate_attrs(data=new_data, excludes=excludes)
        http_method = self._get_update_method()
        result = http_method(path, post_data=new_data, **kwargs)
        if TYPE_CHECKING:
            assert not isinstance(result, requests.Response)
        return result


class CreateMixin(_RestManagerBase):
    _computed_path: Optional[str]
    _from_parent_attrs: Dict[str, Any]
    _obj_cls: Optional[Type[base.RESTObject]]
    _parent: Optional[base.RESTObject]
    _parent_attrs: Dict[str, Any]
    _path: Optional[str]
    openbas: pyobas.OpenBAS

    @exc.on_http_error(exc.OpenBASCreateError)
    def create(
        self, data: Optional[Dict[str, Any]] = None, icon: tuple = None, **kwargs: Any
    ) -> base.RESTObject:
        if data is None:
            data = {}
        self._create_attrs.validate_attrs(data=data)
        # Handle specific URL for creation
        path = kwargs.pop("path", self.path)

        if icon:
            files = {"icon": icon}
        elif icon is False:
            files = {}
        else:
            files = None
        server_data = self.openbas.http_post(
            path, post_data=data, files=files, **kwargs
        )
        if TYPE_CHECKING:
            assert not isinstance(server_data, requests.Response)
            assert self._obj_cls is not None
        return self._obj_cls(self, server_data)



from .attack_pattern import *  # noqa: F401,F403
from .collector import *  # noqa: F401,F403
from .document import *  # noqa: F401,F403
from .endpoint import *  # noqa: F401,F403
from .inject import *  # noqa: F401,F403
from .inject_expectation import *  # noqa: F401,F403
from .injector import *  # noqa: F401,F403
from .kill_chain_phase import *  # noqa: F401,F403
from .me import *  # noqa: F401,F403
from .organization import *  # noqa: F401,F403
from .payload import *  # noqa: F401,F403
from .security_platform import *  # noqa: F401,F403
from .team import *  # noqa: F401,F403
from .user import *  # noqa: F401,F403

__all__ = [name for name in dir() if not name.startswith("_")]



from typing import Any, Dict

from pyobas import exceptions as exc
from pyobas.base import RESTManager, RESTObject


class Document(RESTObject):
    _id_attr = "document_id"


class DocumentManager(RESTManager):
    _path = "/documents"
    _obj_cls = Document

    @exc.on_http_error(exc.OpenBASUpdateError)
    def download(self, document_id: str, **kwargs: Any) -> Dict[str, Any]:
        path = f"{self.path}/" + document_id + "/file"
        result = self.openbas.http_get(path, **kwargs)
        return result

    @exc.on_http_error(exc.OpenBASUpdateError)
    def upsert(
        self, document: Dict[str, Any], file: tuple, **kwargs: Any
    ) -> Dict[str, Any]:
        path = f"{self.path}/upsert"
        result = self.openbas.http_post(
            path, post_data=document, files={"file": file}, **kwargs
        )
        return result



import dataclasses
import json
from typing import TYPE_CHECKING, Any, BinaryIO, Dict, Optional, Union

import requests
from requests import PreparedRequest
from requests.auth import AuthBase
from requests.structures import CaseInsensitiveDict
from requests_toolbelt.multipart.encoder import MultipartEncoder  # type: ignore

from pyobas.backends import protocol


class Auth:
    def __init__(self, token: str):
        self.token = token


class TokenAuth(Auth, AuthBase):
    def __call__(self, r: PreparedRequest) -> PreparedRequest:
        r.headers["Authorization"] = f"Bearer {self.token}"
        return r


@dataclasses.dataclass
class SendData:
    content_type: str
    data: Optional[Union[Dict[str, Any], MultipartEncoder]] = None
    json: Optional[Union[Dict[str, Any], bytes]] = None

    def __post_init__(self) -> None:
        if self.json is not None and self.data is not None:
            raise ValueError(
                f"`json` and `data` are mutually exclusive. Only one can be set. "
                f"json={self.json!r}  data={self.data!r}"
            )


class RequestsResponse(protocol.BackendResponse):
    def __init__(self, response: requests.Response) -> None:
        self._response: requests.Response = response

    @property
    def response(self) -> requests.Response:
        return self._response

    @property
    def status_code(self) -> int:
        return self._response.status_code

    @property
    def headers(self) -> CaseInsensitiveDict[str]:
        return self._response.headers

    @property
    def content(self) -> bytes:
        return self._response.content

    @property
    def reason(self) -> str:
        return self._response.reason

    def json(self) -> Any:
        return self._response.json()


class RequestsBackend(protocol.Backend):
    def __init__(self, session: Optional[requests.Session] = None) -> None:
        self._client: requests.Session = session or requests.Session()

    @property
    def client(self) -> requests.Session:
        return self._client

    @staticmethod
    def prepare_send_data(
        files: Optional[Dict[str, Any]] = None,
        post_data: Optional[Union[Dict[str, Any], bytes, BinaryIO]] = None,
        raw: bool = False,
    ) -> SendData:
        if files is not None:
            json_data = {"input": (None, json.dumps(post_data), "application/json")}
            post_data = {**files, **json_data}
            multipart_encoder = MultipartEncoder(fields=post_data)
            return SendData(
                data=multipart_encoder, content_type=multipart_encoder.content_type
            )

        if raw and post_data:
            return SendData(data=post_data, content_type="application/octet-stream")

        if TYPE_CHECKING:
            assert not isinstance(post_data, BinaryIO)

        return SendData(json=post_data, content_type="application/json")

    def http_request(
        self,
        method: str,
        url: str,
        json: Optional[Union[Dict[str, Any], bytes]] = None,
        data: Optional[Union[Dict[str, Any], MultipartEncoder]] = None,
        params: Optional[Any] = None,
        timeout: Optional[float] = None,
        verify: Optional[Union[bool, str]] = True,
        stream: Optional[bool] = False,
        **kwargs: Any,
    ) -> RequestsResponse:
        """Make HTTP request

        Args:
            method: The HTTP method to call ('get', 'post', 'put', 'delete', etc.)
            url: The full URL
            data: The data to send to the server in the body of the request
            json: Data to send in the body in json by default
            timeout: The timeout, in seconds, for the request
            verify: Whether SSL certificates should be validated. If
                the value is a string, it is the path to a CA file used for
                certificate validation.
            stream: Whether the data should be streamed

        Returns:
            A requests Response object.
        """
        response: requests.Response = self._client.request(
            method=method,
            url=url,
            params=params,
            data=data,
            timeout=timeout,
            stream=stream,
            verify=verify,
            json=json,
            **kwargs,
        )
        return RequestsResponse(response=response)



from dataclasses import dataclass, field
from enum import Enum
from typing import List


class ContractCardinality(str, Enum):
    One: str = "1"
    Multiple: str = "n"


class VariableType(str, Enum):
    String: str = "String"
    Object: str = "Object"


@dataclass
class ContractVariable:
    key: str
    label: str
    type: VariableType
    cardinality: ContractCardinality
    children: List["ContractVariable"] = field(default_factory=list)



from typing import Any, Dict

from pyobas import exceptions as exc
from pyobas.base import RESTManager, RESTObject
from pyobas.mixins import CreateMixin, ListMixin, UpdateMixin
from pyobas.utils import RequiredOptional


class Team(RESTObject):
    _id_attr = "team_id"


class TeamManager(CreateMixin, ListMixin, UpdateMixin, RESTManager):
    _path = "/teams"
    _obj_cls = Team
    _create_attrs = RequiredOptional(
        required=("team_name",),
        optional=("team_description", "team_organization", "team_tags"),
    )

    @exc.on_http_error(exc.OpenBASUpdateError)
    def upsert(self, team: Dict[str, Any], **kwargs: Any) -> Dict[str, Any]:
        path = f"{self.path}/upsert"
        result = self.openbas.http_post(path, post_data=team, **kwargs)
        return result
